# main.py

########################################################################
#  SdiQt -- SDictionary viewer based on Qt library
#  Copyright (C) 2006 Dmitri Brechalov
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  The full license is in the file COPYING, distributed as part of
#  this software.
########################################################################

from qt import *
try:
    from qtpe import *
    useQtopia = True
except:
    useQtopia = False

import version
import utils
import sdict
from filedialog import FileDialog
from browser import Browser

def prepareArticle(title, text):
    import sre
    text = sre.sub('<r>([^<]+)</r>',
                   '<a href="\\1">\\1</a>',
                   text)
    return '<h3>%s</h3>\n\n%s' % (title, text)

class mainForm(QMainWindow):
    def __init__(self,parent = None):
        QMainWindow.__init__(self, parent, '', 0)

        self._error = None
        self.dct = None
        self.fileName = None
        self.dictHistory = {}
        self.history = []

        self.setCaption(version.product + ' ' + version.version)
        if utils.isQtopia():
            iconDir = '/opt/QtPalmtop/pics'
        else:
            iconDir = utils.getProgramPath()
#        self.setIcon(QPixmap(os.path.join(iconDir, 'pymoney.png'), 'PNG'))
        self.setupMainMenu()
        self.setupToolbar()
        self.setupMainArea()
        self.setupSlots()
        self.languageChange()
        self.resize(QSize(640,480).expandedTo(self.minimumSizeHint()))
        try:
            self.openLastDictionary()
        except Exception, details:
            self._error = details


    def openLastDictionary(self):
        self.readLastDictionary()
        if self.fileName:
            self.loadDictionary()

    def readLastDictionary(self):
        try:
            self.fileName = utils.openConfig('r').readline().strip()
        except:
            self.fileName = None

    def writeLastDictionary(self):
        if self.fileName:
            try:
                utils.openConfig('w').write('%s\n' % self.fileName)
            except:
                pass

    def loadDictionary(self):
        try:
            self.dct = sdict.SDictionary(self.fileName)
        except Exception, details:
            self.messageBox('Cannot open file %s!\n%s' % \
                            (self.fileName, details), 'error')
            self.dct = None
            self.fileName = None
        self.history = []
        self.updateForm()

    def close(self, alsoDelete=False):
        #### save last dict path
        if self.fileName:
            self.writeLastDictionary()
        if self.dct:
            self.dct.close()
        rc = QMainWindow.close(self, alsoDelete)
        return rc

    def setupMainMenu(self):
        self.helpAboutAction = QAction(self, "helpAboutAction")
        self.helpAboutQtAction = QAction(self, "helpAboutQtAction")
        self.fileOpen = QAction(self, "fileOpen")
        self.fileExit = QAction(self, "fileExit")

        if utils.isQtopia():
            self.menubar = QPEMenuBar(self,"menubar")
        else:
            self.menubar = QMenuBar(self,"menubar")

        self.fileMenu = QPopupMenu(self)
        self.fileOpen.addTo(self.fileMenu)
        self.fileMenu.insertSeparator()
        self.fileExit.addTo(self.fileMenu)
        self.menubar.insertItem("&File",self.fileMenu,0)

        self.helpMenu = QPopupMenu(self)
        self.helpAboutAction.addTo(self.helpMenu)
        self.helpAboutQtAction.addTo(self.helpMenu)
        self.menubar.insertItem("&?",self.helpMenu,4)

    def setupToolbar(self):
        if utils.isQtopia():
            self.toolBar = QPEToolBar(self)
        else:
            self.toolBar = QToolBar(self)
        try:
            self.setToolBarsMovable(False)
        except:
            pass

        self.dictLbl = QLabel(self.toolBar, "dictLbl")
        self.dictLbl.setText("Dictionary: ")
        self.dictList = QComboBox(0,self.toolBar,"dictList")
        self.dictList.setMinimumSize(QSize(150,0))
        self.toolBar.setStretchableWidget(self.dictList)
        self.btnBack = QPushButton(self.toolBar, 'btnBack')
        self.btnBack.setText('<=')
        self.btnBack.setEnabled(False)

    def setupMainArea(self):
        w = QWidget(self, "w")
        self.setCentralWidget(w)
        mainFormLayout = QHBoxLayout(w,1,2,"mainFormLayout")
        lw = QVBox(w, "lw")
        lw.setMaximumSize(250, lw.maximumHeight())
        lw.setSizePolicy(QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Expanding))
        mainFormLayout.addWidget(lw)
        self.wordSearch = QComboBox(lw, "wordSearch")
        self.wordSearch.setEditable(True)
        self.wordList = QListBox(lw,"wordList")
        rightLayout = QVBoxLayout(None,1,2,"rightLayout")
        self.textView = Browser(w, "textView")
        rightLayout.addWidget(self.textView)
        mainFormLayout.addLayout(rightLayout)
        self.wordSearch.setFocus()

    def setupSlots(self):
        #### main menu
        self.connect(self.helpAboutAction, SIGNAL("activated()"),
                     self.helpAbout)
        self.connect(self.helpAboutQtAction, SIGNAL("activated()"),
                     self.helpAboutQt)
        self.connect(self.fileOpen, SIGNAL("activated()"),
                     self.openDictionary)
        self.connect(self.fileExit, SIGNAL("activated()"),
                     self.close)
        #### searching
        self.connect(self.wordSearch.lineEdit(),
                     SIGNAL("returnPressed()"),
                     self.doSearch)
        self.connect(self.wordList,
                     SIGNAL("highlighted(int)"),
                     self.activateWordList)
        #### hypertext capabilities
        self.connect(self.textView,
                     PYSIGNAL("hyperlinkClicked()"),
                     self.hyperlinkClicked)
        self.connect(self.btnBack,
                     SIGNAL("clicked()"),
                     self.backward)

    def languageChange(self):
        self.helpAboutAction.setText(self.tr("About"))
        self.helpAboutAction.setMenuText(self.tr("&About"))

        self.helpAboutQtAction.setText(self.tr("About Qt"))
        self.helpAboutQtAction.setMenuText(self.tr("About &Qt"))

        self.fileOpen.setText(self.tr("Open dictionary"))
        self.fileOpen.setMenuText(self.tr("&Open"))
        self.fileOpen.setAccel(Qt.CTRL+Qt.Key_O)

        self.fileExit.setText(self.tr("Exit"))
        self.fileExit.setMenuText(self.tr("E&xit"))
        self.fileExit.setAccel(Qt.CTRL+Qt.Key_Q)

    def messageBox(self, msg, title="Warning"):
        QMessageBox.warning(self, title, msg)

    def notImplemented(self, feature):
        self.messageBox('Sorry, %s not implemented yet!' % feature)

    def helpAbout(self):
        QMessageBox.about(self, "About",
                          "<h2>" + version.product + "</h2>" + \
                          "<h3>" + version.description + "</h3>" + \
                          "<p>Version %s<p>" % \
                          (version.version) + \
                          '<nobr>' + version.copyright + '</nobr>' + \
                          '<p><small>' + version.notice + '</small>')

    def helpAboutQt(self):
        QMessageBox.aboutQt(self, "About Qt")
        
    def openDictionary(self):
        import os.path
        if self.fileName:
            dirName = os.path.split(self.fileName)[0]
        else:
            dirName=os.path.expanduser('~')
        if utils.isQtopia():
            dlg = FileDialog(self, 'OpenFile', 'Open Dictionary',
                             dirName = dirName,
                             filter='*.dct')
        else:
            dlg = QFileDialog(dirName, '*.dct', self, 'OpenFile')
        if dlg.exec_loop() and self.fileName:
            self.fileName = str(dlg.selectedFile().utf8())
            self.loadDictionary()

    def updateForm(self):
        if self.dct is None:
            self.dictList.setCurrent(-1)
        else:
            utils.appendHistory(self.dictHistory, self.dictList,
                                self.fileName, self.dct.title)
        self.wordSearch.setEditText('')
        self.btnBack.setEnabled(len(self.history)>1)

    def activateWordList(self, idx):
        self.wordList.setSelected(idx, True)
        self.showWord(str(self.wordList.currentText().utf8()))

    def doSearch(self):
        txt = str(self.wordSearch.currentText().utf8())
        self.wordList.clear()
#         if not txt:
#             return
        for item in self.dct.get_word_list(txt):
            self.wordList.insertItem(QString.fromUtf8(item))
        if self.wordList.count() > 0:
            self.activateWordList(0)

    def showWord(self, word):
        article = prepareArticle(word, self.dct.lookup(word))
        self.textView.setText(QString.fromUtf8(article))
        if not self.history or self.history[-1] != word:
            self.history.append(word)
        self.btnBack.setEnabled(len(self.history)>1)

    def hyperlinkClicked(self, word):
        self.wordSearch.setEditText(word)
        self.doSearch()

    def backward(self):
        if len(self.history) > 1:
            word = self.history.pop()   # the current page
            word = self.history.pop()
            self.wordSearch.setEditText(QString.fromUtf8(word))
            self.doSearch()

# end of main.py
